using System;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.IO;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using System.Security.Cryptography;
namespace CryptoFuncTests
{
    /// <summary>
    /// Summary description for DPAPIFixture
    /// </summary>
    [TestClass]
    public class DPAPIFixtureSystemSource
    {
        private byte[] PlainByteMessage;
        private String PlainStrMessage;
        private const String DPAPISymmetricCryptographyProvider = "DPAPISymmetricCryptographyProvider";
        private const String DPAPISymmetricCryptographyProviderMachineMode = "DPAPISymmetricCryptographyProviderMachineMode";
        private const String MachineModeFile = @"c:\MachineModeDpapi.Txt";
        private const String UserModeFile = @"c:\UserModeDpapi.Txt";
        public DPAPIFixtureSystemSource()
        {
            String PlainStrMessageNotInBase64 = "This is Plain Message";
            PlainByteMessage = System.Text.Encoding.Unicode.GetBytes(PlainStrMessageNotInBase64);
            PlainStrMessage = System.Convert.ToBase64String(PlainByteMessage);
        }
        # region EncryptionSymmetic Tests For DPAPIUserMode
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestDPAPIForNullPlainMessage()
        {
            CryptoHelper.Encrypt(DPAPISymmetricCryptographyProvider, (String)null);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestDPAPIForNullPlainMessageonBytes()
        {
            CryptoHelper.Encrypt(DPAPISymmetricCryptographyProvider, null, (byte[])null);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDPAPIForEmptyPlainMessage()
        {
            CryptoHelper.Encrypt(DPAPISymmetricCryptographyProvider, String.Empty);
        }
        [TestMethod]
        public void TestDPAPIEncryptionForValidInputs()
        {
            String encryptedStrMessage = CryptoHelper.Encrypt(DPAPISymmetricCryptographyProvider, PlainStrMessage);
            Assert.IsTrue(encryptedStrMessage != PlainStrMessage);
        }
        # endregion
        # region EncryptionSymmetic Tests For DPAPISymmetricCryptographyProviderMachineMode
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestDPAPIMachineModeForNullPlainMessage()
        {
            CryptoHelper.Encrypt(DPAPISymmetricCryptographyProviderMachineMode, (String)null);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestDPAPIMachineModeForNullPlainMessageonBytes()
        {
            
            CryptoHelper.Encrypt(DPAPISymmetricCryptographyProviderMachineMode, null, (byte[])null);
        }

        
        [TestMethod]
        public void TestDPAPIMachineModeEncryptionForValidInputs()
        {
            byte[] entropy = new byte[16];
            CryptographyUtility.GetRandomBytes(entropy);
            DpapiSymmetricCryptoProvider prov = new DpapiSymmetricCryptoProvider(DataProtectionScope.LocalMachine, entropy);
            byte[] plainText = new byte[16];
            CryptographyUtility.GetRandomBytes(plainText);

            byte[] cipher = prov.Encrypt(plainText);
            byte[] deciphered = prov.Decrypt(cipher);
            CryptographyUtility.CompareBytes(deciphered, plainText);
        }
        # endregion
        #region DecryptSymmetric Tests For Dpapi User Mode
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDpapiDecryptForNullCipherMessage()
        {
            CryptoHelper.Decrypt(DPAPISymmetricCryptographyProvider, (String)null);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestDpapiDecryptForNullCipherMessageAndNullCryptoInstance()
        {
            CryptoHelper.Decrypt(DPAPISymmetricCryptographyProvider, null, (byte[])null);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDpapiDecryptForEmptyCipherMessage()
        {
            CryptoHelper.Decrypt(DPAPISymmetricCryptographyProvider, String.Empty);
        }
        [TestMethod]
        public void TestDpapiDecryptForValidInputs()
        {
            String encryptedStrMessage = CryptoHelper.Encrypt(DPAPISymmetricCryptographyProvider, PlainStrMessage);
            Assert.AreEqual(CryptoHelper.Decrypt(DPAPISymmetricCryptographyProvider, encryptedStrMessage), PlainStrMessage);
        }
        #endregion
        #region DecryptSymmetric Tests For Dpapi Machine Mode
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDpapiMachineModeDecryptForNullCipherMessage()
        {
            CryptoHelper.Decrypt(DPAPISymmetricCryptographyProviderMachineMode, (String)null);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestDpapiMachineModeDecryptForNullCipherMessageAndNullCryptoInstance()
        {
            CryptoHelper.Decrypt(DPAPISymmetricCryptographyProviderMachineMode, null, (byte[])null);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDpapiMachineModeDecryptForEmptyCipherMessage()
        {
            CryptoHelper.Decrypt(DPAPISymmetricCryptographyProviderMachineMode, String.Empty);
        }

        [TestMethod]
        public void TestDpapiForMachineMode()
        {
            SymmetricCryptoProviderFactory fact = new SymmetricCryptoProviderFactory(new SystemConfigurationSource());
            ISymmetricCryptoProvider prov = fact.Create("DPAPISymmetricCryptographyProviderMachineMode");
            prov.Encrypt(new byte[] { 1, 2, 3 });

        }

       
        # endregion
    }
}
