
using System;
using System.Runtime.InteropServices;
using System.Security.Principal;
using System.Security.Permissions;
using System.Reflection.Emit;
using System.IO;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
[assembly: SecurityPermissionAttribute(SecurityAction.RequestMinimum, UnmanagedCode = true)]
[assembly: PermissionSetAttribute(SecurityAction.RequestMinimum, Name = "FullTrust")]
// Create a 'testuser' with password '#entlib123' and uncomment the two test methods for this test to suceed
namespace CryptoFuncTests
{
    /// <summary>
    /// Summary description for DPAPIFixtureUsingImpersonation
    /// </summary>
    [TestClass]
    public class DPAPIFixtureUsingImpersonation
    {
        private byte[] PlainByteMessage;
        private String PlainStrMessage;
        private const String DPAPISymmetricCryptographyProvider = "DPAPISymmetricCryptographyProvider";
        private const String DPAPISymmetricCryptographyProviderMachineMode = "DPAPISymmetricCryptographyProviderMachineMode";
        [DllImport("advapi32.dll", SetLastError = true)]
        public static extern bool LogonUser(String lpszUsername, String lpszDomain, String lpszPassword, int dwLogonType, int dwLogonProvider, ref IntPtr phToken);
        [DllImport("kernel32.dll", CharSet = System.Runtime.InteropServices.CharSet.Auto)]
        private unsafe static extern int FormatMessage(int dwFlags, ref IntPtr lpSource, int dwMessageId, int dwLanguageId, ref String lpBuffer, int nSize, IntPtr* Arguments);
        [DllImport("kernel32.dll", CharSet = CharSet.Auto)]
        public extern static bool CloseHandle(IntPtr handle);
        [DllImport("advapi32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        public extern static bool DuplicateToken(IntPtr ExistingTokenHandle, int SECURITY_IMPERSONATION_LEVEL, ref IntPtr DuplicateTokenHandle);
        private const String MachineModeFile = @"MachineMode.Txt";
        private const String UserModeFile = @"UserMode.Txt";
        string userName = "testuser";
        string domainName = System.Environment.MachineName;
        string password = "#entlib123";
        
        public DPAPIFixtureUsingImpersonation()
        {
            String PlainStrMessageNotInBase64 = "This is Plain Message";
            PlainByteMessage = System.Text.Encoding.Unicode.GetBytes(PlainStrMessageNotInBase64);
            PlainStrMessage = System.Convert.ToBase64String(PlainByteMessage);
            if (File.Exists(UserModeFile))
            {
                File.Delete(UserModeFile);
            }
            StreamWriter sw1 = new StreamWriter(UserModeFile);
            sw1.WriteLine(Cryptographer.EncryptSymmetric(DPAPISymmetricCryptographyProvider, PlainStrMessage));
            sw1.Close();
            if (File.Exists(MachineModeFile))
            {
                File.Delete(MachineModeFile);
            }
            StreamWriter sw2 = new StreamWriter(MachineModeFile);
            sw2.WriteLine(Cryptographer.EncryptSymmetric(DPAPISymmetricCryptographyProviderMachineMode, PlainStrMessage));
            sw2.Close();
        }
        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize]
        public void Initialize()
        {
        }
        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }
        [TestMethod]
        [PermissionSetAttribute(SecurityAction.Demand, Name = "FullTrust")]
        // create the user testuser with password #entlib123
        public unsafe void TestWithImpersonation()
        {
            IntPtr tokenHandle = new IntPtr(0);
            IntPtr dupeTokenHandle = new IntPtr(0);
            const int LOGON32_PROVIDER_DEFAULT = 0;
            //This parameter causes LogonUser to create a primary token.
            const int LOGON32_LOGON_INTERACTIVE = 2;
            const int SecurityImpersonation = 2;
            tokenHandle = IntPtr.Zero;
            dupeTokenHandle = IntPtr.Zero;
            // Call LogonUser to obtain a handle to an access token.
            bool returnValue = LogonUser(userName, domainName, password, LOGON32_LOGON_INTERACTIVE, LOGON32_PROVIDER_DEFAULT, ref tokenHandle);
            if (false == returnValue)
            {
                int ret = Marshal.GetLastWin32Error();
                //Console.WriteLine("LogonUser failed with error code : {0}", ret);
                //Console.WriteLine("\nError: [{0}] {1}\n", ret, GetErrorMessage(ret));
                int errorCode = 0x5; //ERROR_ACCESS_DENIED
                throw new System.ComponentModel.Win32Exception(errorCode);
            }
            bool retVal = DuplicateToken(tokenHandle, SecurityImpersonation, ref dupeTokenHandle);
            if (false == retVal)
            {
                CloseHandle(tokenHandle);
                return;
            }
            // The token that is passed to the following constructor must 
            // be a primary token in order to use it for impersonation.
            WindowsIdentity newId = new WindowsIdentity(dupeTokenHandle);
            WindowsImpersonationContext impersonatedUser = newId.Impersonate();
            try
            {
                StreamReader sr = new StreamReader(UserModeFile);
                String EncryptedMessage = sr.ReadToEnd();
                sr.Close();
                String DecryptedMessage = Cryptographer.DecryptSymmetric(DPAPISymmetricCryptographyProvider, EncryptedMessage);
                Assert.IsTrue(DecryptedMessage == PlainStrMessage);
                throw new Exception("Impersonation Failed");
            }
            catch
            {
                // This test suceeded.
            }
            finally
            {
                impersonatedUser.Undo();
            }
            // Check the identity.
            // Free the tokens.
            if (tokenHandle != IntPtr.Zero)
                CloseHandle(tokenHandle);
            if (dupeTokenHandle != IntPtr.Zero)
                CloseHandle(dupeTokenHandle);
        }
        [TestMethod]
        //create the user testuser with password #entlib123
        public void TestWithoutImpersonation()
        {
            using (StreamReader sr = new StreamReader(UserModeFile))
            {
                String EncryptedMessage = sr.ReadToEnd();
                String DecryptedMessage = Cryptographer.DecryptSymmetric(DPAPISymmetricCryptographyProvider, EncryptedMessage);
                Assert.IsTrue(DecryptedMessage == PlainStrMessage);
            }
        }
    }
}
