using System;
using System.Text;
using System.Diagnostics;
using System.Configuration; 
using System.Collections.Generic;
using System.Data.Common;
using System.Messaging;
using System.Management;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.Formatters;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.Database;
using Microsoft.Practices.EnterpriseLibrary.Logging.Database.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation.Configuration;  
using CustomClasses;

namespace ConfigurationSourceTests
{
    /// <summary>
    /// Summary description for DictionarySourceGenericTestFixture
    /// </summary>
    [TestClass]
    public class DictionarySourceGenericTestFixture
    {
        private DictionaryConfigurationSource source;
        private bool logged = false;
        private string loggedMessage = "";

        public DictionarySourceGenericTestFixture()
        {
            source = LoggingConfigurationSource.BuildConfigurationSource();
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            TraceSourceData msmqSource = (TraceSourceData)setting.TraceSources.Get("MsmqCategory");
            TraceSourceData dbSource = (TraceSourceData)setting.TraceSources.Get("DatabaseCategory");
            TraceSourceData defaultsource = (TraceSourceData)setting.TraceSources.Get("General");    
            setting.SpecialTraceSources = new SpecialTraceSourcesData(msmqSource, dbSource, defaultsource);
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        #region Special Source Tests
        [TestMethod]
        public void NotProcessedTraceSourceTest()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            TraceSourceData actualSource = setting.SpecialTraceSources.AllEventsTraceSource;
            setting.SpecialTraceSources.AllEventsTraceSource = new TraceSourceData();    
            LogEntry log = new LogEntry();
            log.Categories.Add("NotProcessedTraceSource");
            log.Message = "To test the message is logged to NotProcessedTraceSource";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
               writer.Write(log);
            }
            DatabaseProviderFactory Dbfactory = new DatabaseProviderFactory(source);
            Database db = Dbfactory.Create("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 Message FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.AreEqual(messageContents, log.Message);
            setting.SpecialTraceSources.AllEventsTraceSource = actualSource;   
        }

        [TestMethod]
        public void ErrorTraceSourceTest()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            FormattedDatabaseTraceListenerData dbListener = (FormattedDatabaseTraceListenerData)setting.TraceListeners.Get("Database Trace Listener");
            string dbInstanceName = dbListener.DatabaseInstanceName;
            dbListener.DatabaseInstanceName = "InvalidInstance";

            LogEntry entry = new LogEntry();
            entry.Categories.Add("DatabaseCategory");
            entry.Message = "To test the Error Trace Source";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                using (EventLog log = new EventLog(HelperClass.GetEventLogName()))
                {
                    int initialCount = log.Entries.Count;
                    writer.Write(entry);
                    int finalCount = log.Entries.Count;
                    Assert.IsTrue(finalCount - initialCount == 1);
                    Assert.IsTrue(log.Entries[finalCount - 1].Message.Contains(entry.Message));
                }
            }
            dbListener.DatabaseInstanceName = dbInstanceName;
        }

        [TestMethod]
        public void AllEventsTraceSourceTest()
        {
            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(new LogEntry("To test AllEventsTraceSource", "General", 1, 1, TraceEventType.Critical, "", null));
                }
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string)message.Body;
                Assert.IsTrue(content.Contains("To test AllEventsTraceSource"));
            }
        }

      
        #endregion

        #region Additional Tests
        [TestMethod]
        public void InstrumentationEventLogEnabledTest()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            FormattedDatabaseTraceListenerData dbListener = (FormattedDatabaseTraceListenerData)setting.TraceListeners.Get("Database Trace Listener");
            string dbInstanceName = dbListener.DatabaseInstanceName;
            dbListener.DatabaseInstanceName = "InvalidInstance";
            TraceSourceData actualSource = setting.SpecialTraceSources.ErrorsTraceSource;
            setting.SpecialTraceSources.ErrorsTraceSource = setting.TraceSources.Get("DatabaseCategory");

            LogEntry entry = new LogEntry();
            entry.Categories.Add("DatabaseCategory");
            entry.Message = "To test if the error is logged to the event when instrumentation is on";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (EventLog log = new EventLog("Application"))
            {
                int initialCount = log.Entries.Count;
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(entry);
                }
                int finalCount = log.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 1);
                Assert.IsTrue(log.Entries[finalCount - 1].Message.Contains("Failure while sending log entry to the TraceListeners"));
                Assert.IsTrue(log.Entries[finalCount-1].Source.Equals("Enterprise Library Logging"));      
            }
            dbListener.DatabaseInstanceName = dbInstanceName;
            setting.SpecialTraceSources.ErrorsTraceSource = actualSource;
        }

        [TestMethod]
        public void InstrumentationEventLogDisabledTest()
        {
            InstrumentationConfigurationSection instrumentation = (InstrumentationConfigurationSection)source.GetSection("instrumentationConfiguration");
            instrumentation.EventLoggingEnabled = false;
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            FormattedDatabaseTraceListenerData dbListener = (FormattedDatabaseTraceListenerData)setting.TraceListeners.Get("Database Trace Listener");
            string dbInstanceName = dbListener.DatabaseInstanceName;
            dbListener.DatabaseInstanceName = "InvalidInstance";
            TraceSourceData actualSource = setting.SpecialTraceSources.ErrorsTraceSource;
            setting.SpecialTraceSources.ErrorsTraceSource = setting.TraceSources.Get("DatabaseCategory");

            LogEntry entry = new LogEntry();
            entry.Categories.Add("DatabaseCategory");
            entry.Message = "To test the LoggingFailureLoggingErrorEvent";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (EventLog log = new EventLog("Application"))
            {
                int initialCount = log.Entries.Count;
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(entry);
                }
                int finalCount = log.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 0);
            }
            dbListener.DatabaseInstanceName = dbInstanceName;
            setting.SpecialTraceSources.ErrorsTraceSource = actualSource;
            instrumentation.EventLoggingEnabled = true; 
        }

        [TestMethod]
        public void InstrumentationWmiEnabledTest()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            FormattedDatabaseTraceListenerData dbListener = (FormattedDatabaseTraceListenerData)setting.TraceListeners.Get("Database Trace Listener");
            string dbInstanceName = dbListener.DatabaseInstanceName;
            dbListener.DatabaseInstanceName = "InvalidInstance";
            TraceSourceData actualSource = setting.SpecialTraceSources.ErrorsTraceSource;
            setting.SpecialTraceSources.ErrorsTraceSource = setting.TraceSources.Get("DatabaseCategory");
  
            LogEntry entry = new LogEntry();
            entry.Categories.Add("DatabaseCategory");
            entry.Message = "To test the LoggingFailureLoggingErrorEvent";
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM LoggingFailureLoggingErrorEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start();
            logged = false;
            loggedMessage = "";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(entry);
            }
            System.Threading.Thread.Sleep(1000);
            watcher.Stop();
            watcher.Dispose();
            Assert.IsTrue(logged);
            Assert.IsTrue(loggedMessage.Contains("Failure while sending log entry to the TraceListeners"));     
            dbListener.DatabaseInstanceName = dbInstanceName;
            setting.SpecialTraceSources.ErrorsTraceSource = actualSource;     
        }

        [TestMethod]
        public void InstrumentationWmiDisabledTest()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            FormattedDatabaseTraceListenerData dbListener = (FormattedDatabaseTraceListenerData)setting.TraceListeners.Get("Database Trace Listener");
            string dbInstanceName = dbListener.DatabaseInstanceName;
            dbListener.DatabaseInstanceName = "InvalidInstance";
            TraceSourceData actualSource = setting.SpecialTraceSources.ErrorsTraceSource;
            setting.SpecialTraceSources.ErrorsTraceSource = setting.TraceSources.Get("DatabaseCategory");
            InstrumentationConfigurationSection instrumentation = (InstrumentationConfigurationSection)source.GetSection("instrumentationConfiguration");
            instrumentation.WmiEnabled = false;

            LogEntry entry = new LogEntry();
            entry.Categories.Add("DatabaseCategory");
            entry.Message = "To test the LoggingFailureLoggingErrorEvent";
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM LoggingFailureLoggingErrorEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start();
            logged = false;
            loggedMessage = "";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(entry);
            }
            System.Threading.Thread.Sleep(1000);
            watcher.Stop();
            watcher.Dispose();
            Assert.IsFalse(logged);
            Assert.IsFalse(loggedMessage.Contains("Failure while sending log entry to the TraceListeners"));
            dbListener.DatabaseInstanceName = dbInstanceName;
            setting.SpecialTraceSources.ErrorsTraceSource = actualSource;
            instrumentation.WmiEnabled = true; 
        }

        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {
            loggedMessage = args.NewEvent.GetText(TextFormat.Mof);
            logged = true;
        }

        [TestMethod]
        public void InstrumentationPerfCounterEnabledTest()
        {
            string counterName = "Trace Listener Entries Written/sec";
            int initialCount = HelperClass.GetCounterValue(counterName, "Total");
            LogEntry entry = new LogEntry();
            entry.Severity = TraceEventType.Error;
            entry.Message = "";
            entry.Categories.Add("DatabaseCategory");
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(entry);
            }
            int loggedCount = HelperClass.GetCounterValue(counterName, "Total");
            Assert.IsTrue((loggedCount - initialCount) == 2);
        }

        [TestMethod]
        public void InstrumentationPerfCounterDisabledTest()
        {
            InstrumentationConfigurationSection instrumentation = (InstrumentationConfigurationSection)source.GetSection("instrumentationConfiguration");
            instrumentation.PerformanceCountersEnabled = false;
            string counterName = "Trace Listener Entries Written/sec";
            int initialCount = HelperClass.GetCounterValue(counterName, "Total");
            LogEntry entry = new LogEntry();
            entry.Severity = TraceEventType.Error;
            entry.Message = "";
            entry.Categories.Add("DatabaseCategory");
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(entry);
            }
            int loggedCount = HelperClass.GetCounterValue(counterName, "Total");
            Assert.IsTrue((loggedCount - initialCount) == 0);
            instrumentation.PerformanceCountersEnabled = true; 
        }
        #endregion
    }
}
